#!/usr/bin/env node
'use strict';

var hardRejection = require('hard-rejection');
var sade = require('sade');
var path = require('path');
var findCacheDir = require('find-cache-dir');
var makeDir = require('make-dir');
var del = require('del');
var execSh = require('exec-sh');
var flatCache = require('flat-cache');
var lodash = require('lodash');
var dateFns = require('date-fns');
var hasha = require('hasha');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var hardRejection__default = /*#__PURE__*/_interopDefaultLegacy(hardRejection);
var sade__default = /*#__PURE__*/_interopDefaultLegacy(sade);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var findCacheDir__default = /*#__PURE__*/_interopDefaultLegacy(findCacheDir);
var makeDir__default = /*#__PURE__*/_interopDefaultLegacy(makeDir);
var del__default = /*#__PURE__*/_interopDefaultLegacy(del);
var execSh__default = /*#__PURE__*/_interopDefaultLegacy(execSh);
var flatCache__default = /*#__PURE__*/_interopDefaultLegacy(flatCache);
var hasha__default = /*#__PURE__*/_interopDefaultLegacy(hasha);

var name = "debounce-cmd";
var version = "0.2.0";

function getCacheDirectoryPath(relativeCacheDirectoryPath) {
  if (relativeCacheDirectoryPath) {
    return path__default["default"].resolve(process.cwd(), relativeCacheDirectoryPath);
  }

  const resolvedCacheDirectory = findCacheDir__default["default"]({
    name
  });

  if (!resolvedCacheDirectory) {
    throw Error('Could not find cache directory. Please provide a cache directory manually.');
  }

  return resolvedCacheDirectory;
}
function createCache(relativeCacheDirectoryPath) {
  if (relativeCacheDirectoryPath) {
    const absoluteCacheDirectoryPath = makeDir__default["default"].sync(relativeCacheDirectoryPath);
    return absoluteCacheDirectoryPath;
  }

  const resolvedCachePath = findCacheDir__default["default"]({
    name,
    create: true
  });

  if (!resolvedCachePath) {
    throw Error('Could not find cache directory. Please provide a cache directory manually.');
  }

  return resolvedCachePath;
}

function showCacheDirectory(relativeCacheDirectory) {
  console.log(getCacheDirectoryPath(relativeCacheDirectory));
}

function clearCacheDirectory(relativeCacheDirectory) {
  const deletedPaths = del__default["default"].sync(getCacheDirectoryPath(relativeCacheDirectory));

  if (deletedPaths.length === 0) {
    console.log('No cache to clear');
  } else if (deletedPaths.length === 1) {
    console.log(`Deleted: ${deletedPaths[0]}`);
  } else {
    console.log('Deleted:\n', deletedPaths.join('\n'));
  }
}

function getFilePaths(relativeFilePaths) {
  const cwd = process.cwd();
  return relativeFilePaths.map(file => path__default["default"].resolve(cwd, file)).sort();
}

function getDuration(durationString) {
  if (!/^(\d+[a-z]+)+$/gi.test(durationString)) {
    throw Error(`Invalid duration: ${durationString}`);
  }

  const duration = {
    years: undefined,
    months: undefined,
    weeks: undefined,
    days: undefined,
    hours: undefined,
    minutes: undefined,
    seconds: undefined
  };
  durationString.match(/\d+[a-z]+/gi).forEach(durationPart => {
    const [, stringQuantity, unitShort] = /(\d+)([a-z]+)/gi.exec(durationPart);
    const quantity = Number(stringQuantity);
    const unitLong = {
      y: 'years',
      mo: 'months',
      w: 'weeks',
      d: 'days',
      h: 'hours',
      m: 'minutes',
      s: 'seconds'
    }[unitShort];

    if (Number.isNaN(quantity) || !unitLong) {
      throw Error(`Invalid duration part: ${durationPart}`);
    }

    if (duration[unitLong] !== undefined) {
      throw Error(`Duration with unit ${unitLong} was supplied multiple times`);
    }

    duration[unitLong] = quantity;
  });
  return duration;
}

function getCacheKey({
  duration,
  filePaths,
  command
}) {
  return ['cwd:' + process.cwd(), 'cmd:' + command, duration && 'time:' + JSON.stringify(duration), filePaths.length !== 0 && 'files:' + filePaths.join(',')].filter(Boolean).join(' ---');
}

async function getFileHashes(filePaths) {
  return Object.fromEntries(await Promise.all(filePaths.map(filePath => hasha__default["default"].fromFile(filePath, {
    algorithm: 'md5'
  }).then(hash => [filePath, hash]))));
}

async function debounceCommand({
  relativeCacheDirectory,
  debounceByTime,
  debounceByFiles,
  command
}) {
  if (!debounceByTime && debounceByFiles.length === 0) {
    await execSh__default["default"].promise(command);
    return;
  }

  const cache = flatCache__default["default"].load('commands-cache.json', createCache(relativeCacheDirectory));
  const filePaths = getFilePaths(debounceByFiles);
  const duration = debounceByTime ? getDuration(debounceByTime) : undefined;
  const cacheKey = getCacheKey({
    duration,
    filePaths,
    command
  });
  const cacheData = cache.getKey(cacheKey);
  const fileHashes = filePaths.length === 0 ? undefined : await getFileHashes(filePaths);
  const currentDate = new Date();
  const areFileHashesEqual = lodash.isEqual(cacheData == null ? void 0 : cacheData.fileHashes, fileHashes);

  const isCurrentTimeInDebounce = (() => {
    if (!duration) {
      return true;
    }

    const lastRun = cacheData == null ? void 0 : cacheData.lastRun;

    if (!lastRun) {
      return false;
    }

    return dateFns.isAfter(dateFns.add(new Date(lastRun), duration), currentDate);
  })();

  if (areFileHashesEqual && isCurrentTimeInDebounce) {
    return;
  }

  cache.setKey(cacheKey, {
    lastRun: currentDate,
    fileHashes
  });
  cache.save(true);
  await execSh__default["default"].promise(command);
}

hardRejection__default["default"]();
const program = sade__default["default"]('debounce-cmd');
program.version(version).describe('Debounce a command based on various factors').option('-c, --cache-dir', 'Cache directory to use (default: .cache/debounce-cmd in nearest node_modules)');
program.command('run <command>', 'Run debounced command (if no <command> provided, this is the default)', {
  default: true
}).option('-f, --file', 'Run command only when file content changes').option('-t, --time', 'Run command only after specified time (unit with s,m,h,d,w,mo,y)').example('run "echo ran this command" --time 20s').example('run "yarn install" --file yarn.lock').example('run "yarn install" --file yarn.lock --cache-dir .config/cache').example('run "yarn install" --time 1mo --file yarn.lock --file package.json').action((command, options) => {
  const cacheDirectory = options['cache-dir'];
  const time = options.time;
  const file = options.file;
  const files = file === undefined ? [] : Array.isArray(file) ? file : [file];

  if (typeof command !== 'string') {
    throw Error('Invalid <command> supplied');
  }

  if (cacheDirectory !== undefined && typeof cacheDirectory !== 'string') {
    throw Error('Invalid --cache-dir supplied');
  }

  if (time !== undefined && typeof time !== 'string') {
    throw Error('Invalid --time supplied');
  }

  if (files.some(file => typeof file !== 'string')) {
    throw Error('Invalid --file supplied');
  }

  debounceCommand({
    relativeCacheDirectory: cacheDirectory,
    command,
    debounceByTime: time,
    debounceByFiles: files
  });
});
program.command('cache dir', 'Show cache directory path used by debounce-cmd').action(options => {
  const cacheDirectory = options['cache-dir'];

  if (cacheDirectory !== undefined && typeof cacheDirectory !== 'string') {
    throw Error('Invalid --cache-dir supplied');
  }

  showCacheDirectory(cacheDirectory);
});
program.command('cache clear', 'Clear cache used by debounce-cmd').action(options => {
  const cacheDirectory = options['cache-dir'];

  if (cacheDirectory !== undefined && typeof cacheDirectory !== 'string') {
    throw Error('Invalid --cache-dir supplied');
  }

  clearCacheDirectory(cacheDirectory);
});
program.parse(process.argv);
//# sourceMappingURL=cli.cjs.map
